#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'Optimizer'
__author__ = 'JieYuan'
__mtime__ = '19-3-18'
"""
import numpy as np
from ..cv import LGBMClassifierCV
from .Optimizer import Optimizer


class LGBMOptimizer(Optimizer):
    """https://www.jianshu.com/p/1100e333fcab"""

    def __init__(self, X, y, cv=5, workers=5, random_state=None, params_bounds=None):
        super().__init__(X, y, cv, workers, random_state, params_bounds)
        self.params_bounds['n_estimators'] = 3000

    def objective(self, **params):
        """重写目标函数"""

        # 纠正参数类型
        for p in ('max_depth', 'depth', 'num_leaves', 'subsample_freq', 'min_child_samples'):
            if p in params:
                params[p] = int(np.round(params[p]))
        _params = {**self.params_bounds, **params}

        # 核心逻辑
        self.clf = LGBMClassifierCV(_params, self.cv, self.random_state)
        return self.clf.fit(self.X, self.y, early_stopping_rounds=300, verbose=0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : sk_cv
# @Time         : 2019-09-16 11:32
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


from sklearn.datasets import make_classification
from tql.algo_ml.opt_params import Optimizer
X, y = make_classification(
    n_samples=1000,
    n_features=45,
    n_informative=12,
    n_redundant=7,
    random_state=134985745,
)


opt = Optimizer(X, y)
print(opt.maximize(1))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : XGBOptimizer
# @Time         : 2019-09-16 11:58
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 
import numpy as np
from .Optimizer import Optimizer
from catboost import CatBoostClassifier
from sklearn.model_selection import cross_val_score


class CatBoostOptimizer(Optimizer):

    def __init__(self, X, y, cv=5, workers=5, random_state=None, params_bounds=None):
        super().__init__(X, y, cv, workers, random_state, params_bounds)
        self.params_bounds = params_bounds if params_bounds else \
            dict(n_estimators=100,
                 loss_function="Logloss",
                 eval_metric="AUC",
                 task_type="GPU",
                 learning_rate=(0.001, 1),
                 l2_leaf_reg=(0, 100),
                 random_seed=666,
                 od_type="Iter",
                 depth=(2, 16),
                 early_stopping_rounds=300,
                 border_count=64,
                 has_time=True)

    def objective(self, **params):
        """重写目标函数"""

        # 纠正参数类型
        for p in ('max_depth', 'depth', 'num_leaves', 'subsample_freq', 'min_child_samples'):
            if p in params:
                params[p] = int(np.round(params[p]))
        _params = {**self.params_bounds, **params}

        # 核心逻辑
        self.clf = CatBoostClassifier(**_params)
        _ = cross_val_score(self.clf, self.X, self.y, scoring='roc_auc', cv=5, n_jobs=5).mean()
        return _



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : XGBOptimizer
# @Time         : 2019-09-16 11:58
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

import numpy as np
from xgboost import XGBClassifier
from sklearn.model_selection import cross_val_score

from .Optimizer import Optimizer


class XGBOptimizer(Optimizer):

    def __init__(self, X, y, cv=5, workers=5, random_state=None, params_bounds=None):
        super().__init__(X, y, cv, workers, random_state, params_bounds)

        self.params_bounds = params_bounds if params_bounds else \
            dict(n_estimators=100,
                 max_depth=(5, 16),
                 learning_rate=(0.001, 1),
                 max_delta_step=(0, 2), # 极端不平衡的时候有用

                 gamma=(0.001, 1),  # 描述分裂的最小 gain, 控制树的有用的分裂
                 min_child_weight=(0.001, 100),  # 决定最小叶子节点样本权重和,使一个结点分裂的最小权值之和, 避免过拟合

                 subsample=(0.1, 1),
                 colsample_bytree=(0.1, 1),  # 每棵树的列数
                 # colsample_bylevel=(0.3, 1),  # 每一层的列数

                 reg_alpha=(0, 6),
                 reg_lambda=(0, 6),

                 scale_pos_weight=1,

                 random_state=666,
                 n_jobs=-1,
                 verbosity=-1)

    def objective(self, **params):
        """重写目标函数"""

        # 纠正参数类型
        for p in ('max_depth', 'depth', 'num_leaves', 'subsample_freq', 'min_child_samples'):
            if p in params:
                params[p] = int(np.round(params[p]))
        _params = {**self.params_bounds, **params}

        # 核心逻辑
        self.clf = XGBClassifier(**_params)
        _ = cross_val_score(self.clf, self.X, self.y, scoring='roc_auc', cv=self.cv, n_jobs=self.workers).mean()
        return _



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : UnderSamplerEmsemble
# @Time         : 2019-09-23 13:13
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :

import time
import numpy as np
from sklearn.model_selection import KFold
from ..cv import LGBMClassifierCV


class UnderSamplerEnsemble(object):

    def __init__(self, params=None, cv=5, random_state=None):
        self.oof = LGBMClassifierCV(params, cv, random_state)

    def fit(self, X_p, X_n, X_test, cv=5, random_state=666, pred_ranking=False):
        kf = KFold(cv, True, random_state)

        self.oof_preds = []
        self.oof_scores = []
        for n_fold, (_, idx) in enumerate(kf.split(X_n)):
            print(f"UnderSamplerEnsemble {n_fold + 1} started at {time.ctime()}")
            X = np.r_[X_n[idx], X_p]
            y = np.r_[[0] * len(idx), [1] * len(X_p)]

            _ = self.oof.fit(X, y, X_test)
            self.oof_scores.append(_)
            self.oof_preds.append(self.oof.oof_test_rank if pred_ranking else self.oof.oof_test)

        return np.row_stack(self.oof_preds)
